from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.client import Client, RequestFactory
from django.contrib.sites.models import Site

from contact_form.models import Message, Department, Subject


class ContactFormCase(TestCase):
    urls = 'contact_form.tests.urls'

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        self.site_foo = Site.objects.get(id=1)
        self.user_foo = User.objects.create(username='foo', password='bar', email='foo@test.com')
        self.department_foo = Department.objects.create(
            name='foo department',
            email='department@test.com',
            site=self.site_foo
        )
        self.subject_foo = Subject.objects.create(
            title='test subject title',
            department=self.department_foo,
            site=self.site_foo
        )
        self.message_foo = Message.objects.create(
            sender_name='foo',
            sender_email='foo@test.com',
            subject=self.subject_foo,
            message='test message text',
            ip='127.0.0.1')

    def tearDown(self):
        self.user_foo.delete()
        self.message_foo.delete()
        self.subject_foo.delete()
        self.department_foo.delete()



from __future__ import unicode_literals

from django.conf import settings
from django.urls import reverse_lazy

from django import VERSION
if VERSION >= (2, 0):
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _


class DjeuAppConf:

    USE_CAPTCHA = getattr(settings, 'CONTACT_FORM_USE_CAPTCHA', False)
    USE_RECAPTCHA = getattr(settings, 'CONTACT_FORM_USE_RECAPTCHA', False)
    USE_SIGNALS = getattr(settings, 'CONTACT_FORM_USE_SIGNALS', False)
    SUCCESS_URL = getattr(settings, 'CONTACT_FORM_SUCCESS_URL', reverse_lazy('contact_form'))
    USE_SITES = getattr(settings, 'CONTACT_FORM_USE_SITES', True)
    FILTER_SENDER_NAME = getattr(settings, 'CONTACT_FORM_FILTER_SENDER_NAME', True)
    FILTER_MESSAGE = getattr(settings, 'CONTACT_FORM_FILTER_MESSAGE', True)
    ALLOWED_MESSAGE_TAGS = getattr(settings, 'CONTACT_FORM_ALLOWED_MESSAGE_TAGS', [])
    STRIP_MESSAGE = getattr(settings, 'CONTACT_FORM_STRIP_MESSAGE', False)
    LINKS_NOT_ALLOWED = getattr(settings, 'CONTACT_FORM_LINKS_NOT_ALLOWED', False)
    VALID_MESSAGE = getattr(
        settings,
        'CONTACT_FORM_VALID_MESSAGE',
        _('Your message is submitted')
    )

    INVALID_MESSAGE = getattr(
        settings,
        'CONTACT_FORM_INVALID_MESSAGE',
        _('Something went wrong, your message was not submitted!')
    )

    USE_USERNAME = getattr(settings, 'CONTACT_FORM_USE_USERNAME', True)
    USERNAME_FIELD = getattr(settings, 'CONTACT_FORM_USERNAME_FIELD', 'username')
    USE_USER_EMAIL = getattr(settings, 'CONTACT_FORM_USE_USER_EMAIL', True)
    USER_EMAIL_FIELD = getattr(settings, 'CONTACT_FORM_USER_EMAIL_FIELD', 'email')

    SENDER_NAME_MAX_LENGTH = getattr(settings, 'CONTACT_FORM_SENDER_NAME_MAX_LENGTH', 80)
    SUBJECT_MAX_LENGTH = getattr(settings, 'CONTACT_FORM_SUBJECT_MAX_LENGTH', 80)
    MESSAGE_MAX_LENGTH = getattr(settings, 'CONTACT_FORM_MESSAGE_MAX_LENGTH', 4096)
    MESSAGE_MIN_LENGTH = getattr(settings, 'CONTACT_FORM_MESSAGE_MIN_LENGTH', 15)

    DEPARTMENT_NAME_MAX_LENGTH = getattr(settings, 'CONTACT_FORM_DEPARTMENT_NAME_MAX_LENGTH', 80)
    DEPARTMENT_PHONE_MAX_LENGTH = getattr(settings, 'CONTACT_FORM_DEPARTMENT_PHONE_MAX_LENGTH', 20)

    SUBJECTS_ORDER_BY = getattr(settings, 'CONTACT_FORM_SUBJECTS_ORDER_BY', 'order')

    FORM_MODULE = getattr(settings, 'CONTACT_FORM_FORM_MODULE', 'contact_form.forms.uniform')

    FORM_CAPTCHA_MODULE = getattr(
        settings, 'CONTACT_FORM_FORM_CAPTCHA_MODULE', 'contact_form.forms.uniform'
    )

    FORM_TEMPLATE = getattr(settings, 'CONTACT_FORM_FORM_TEMPLATE', 'contact_form/form.html')

    RENDER_CANCEL_BUTTON = getattr(settings, 'CONTACT_FORM_RENDER_CANCEL_BUTTON', False)

    FORM_LABEL_CLASS = getattr(settings, 'CONTACT_FORM_FORM_LABEL_CLASS', '')
    FORM_FIELD_CLASS = getattr(settings, 'CONTACT_FORM_FORM_FIELD_CLASS', '')

    class Meta:
        prefix = 'contact_form'



from django.db.models import fields
from djeu import forms
from djeu.utils.jdcal import jcal2jd, jd2gcal, gcal2jd, jd2jcal


class JulianDateField(fields.DateField):

    def formfield(self, **kwargs):
        return fields.Field.formfield(self, **{
            'form_class': forms.JulianDateField,
            **kwargs,
        })

    def value_to_string(self, obj):
        val = self.value_from_object(obj)
        julian_year, julian_month, julian_day, _ = jd2jcal(*gcal2jd(val.year, val.month, val.day))
        return '' if val is None else f'{julian_day}.{julian_month}.{julian_year}({val.isoformat()})'



from datetime import date
from django import forms
from django.conf import settings
from django.shortcuts import render

from .date_picker_input import DatePickerInput


class PartialDatePickerInput(DatePickerInput):
    
    def __init__(self, *args, use_admin_site_jquery=False, **kwargs):
        super(PartialDatePickerInput, self).__init__(*args, **kwargs)
        self.use_admin_site_jquery = use_admin_site_jquery

    def get_context(self, name, value, attrs):
        context = super(PartialDatePickerInput,
                        self).get_context(name, value, attrs)
        #  Override string value
        context['widget']['value'] = value if isinstance(
            value, str) else context['widget']['value']
        return context

    @staticmethod
    def prepare_value(value, default_value='-') -> str:
        if not value:
            return default_value
        if isinstance(value, date):
            value = value.isoformat()

        iso_value = value.split('-')
        year = iso_value[0] if len(iso_value) > 0 else '____'
        month = iso_value[1] if len(iso_value) > 1 else '__'
        day = iso_value[2] if len(iso_value) > 2 else '__'
        return f'{day}.{month}.{year}'

    @staticmethod
    def python_value(value) -> str:
        if value == '-' or not value:
            return ''
        date = value.split('.')
        day = date[0] if len(date) > 0 else '__'
        month = date[1] if len(date) > 1 else '__'
        year = date[2] if len(date) > 2 else '____'
        return f'{year}-{month}-{day}'

    def render(self, name, value, attrs=None, renderer=None):
        """Render the widget as an HTML string."""
        if hasattr(self, 'read_only') and self.read_only:
            return self.prepare_value(value)
        return super(PartialDatePickerInput, self).render(name, value, attrs=attrs, renderer=renderer)

    @property
    def media(self):
        extra = '' if settings.DEBUG else '.min'
        if self.use_admin_site_jquery:
            return forms.Media(
                js=(
                    'admin/js/vendor/jquery/jquery%s.js' % extra,
                    'jquery-ui/jquery-ui%s.js' % extra,
                    # 'djeu/js/jquery-datepicker.js',
                    'djeu/js/i18n/datepicker-ru-RU-partial.js',
                    'djeu/js/jquery-datepicker.init.js',
                ) + (
                    'admin/js/jquery.init.js',
                ),
                css={
                    'screen': (
                        'jquery-ui/jquery-ui.min.css',
                    ),
                },
            )
        else:
            return forms.Media(
                js=(
                    'jquery-ui/jquery-ui%s.js' % extra,
                    # 'djeu/js/jquery-datepicker.js',
                    'djeu/js/i18n/datepicker-ru-RU-partial.js',
                    'djeu/js/jquery-datepicker.init.js',
                ),
                css={
                    'screen': (
                        'jquery-ui/jquery-ui.min.css',
                    ),
                },
            )            



from django.db.models.fields.related import ForeignKey, RelatedField
from djeu.forms import NationalityModelChoiceField


class NationalityForeignKey(ForeignKey):

    def __init__(self, to, on_delete, related_name=None, related_query_name=None,
                 limit_choices_to=None, parent_link=False, to_field=None,
                 db_constraint=True, depends=None, key_data_components=None,
                 **kwargs):
        super(NationalityForeignKey, self).__init__(to, on_delete, related_name, related_query_name,
                                                 limit_choices_to, parent_link, to_field, db_constraint,
                                                 **kwargs)
        self.depends = depends
        self.additional_fields = ()

    @property
    def key_data_components(self):
        return (self.name, )

    def formfield(self, form_class=None, choices_form_class=None, **kwargs):
        # self.remote_field.field.objects.all()
        # self.remote_field._default_manager.using(using)
        queryset = self.remote_field.model.objects.all()

        defaults = {
            'django_field': self,
            'form_class': NationalityModelChoiceField,
            'queryset': queryset,
            **kwargs,
        }
        # If initial is passed in, it's a list of related objects, but the
        # MultipleChoiceField takes a list of IDs.
        if defaults.get('initial') is not None:
            initial = defaults['initial']
            if callable(initial):
                initial = initial()
            defaults['initial'] = [i.pk for i in initial]
        return super(RelatedField, self).formfield(**defaults)
